@@ -5,10 +5,11 @@ from __future__ import division
 import numpy as np
 
 from library import gibbs
-from gsw.utilities import match_args_return, strip_mask
-from constants import Kelvin, db2Pascal, P0, SSO, cp0, R, sfac
-from conversions import pt_from_CT, pt_from_t, pt0_from_t, molality_from_SA
 from absolute_salinity_sstar_ct import CT_from_t
+from gsw.utilities import match_args_return, strip_mask
+from conversions import pt_from_CT, pt_from_t, pt0_from_t
+from constants import Kelvin, db2Pascal, P0, SSO, cp0, R, sfac, M_S
+
 __all__ = [
            'rho_t_exact',
            'pot_rho_t_exact',
@@ -42,6 +43,8 @@ __all__ = [
            'osmotic_pressure_t_exact'
           ]
 
+n0, n1, n2 = 0, 1, 2
+
 
 @match_args_return
 def Helmholtz_energy_t_exact(SA, t, p):
@@ -96,10 +99,8 @@ def Helmholtz_energy_t_exact(SA, t, p):
     2011-03-29. Trevor McDougall
     """
 
-    n0, n1 = 0, 1
-    g000 = gibbs(n0, n0, n0, SA, t, p)
-    g001 = gibbs(n0, n0, n1, SA, t, p)
-    return (g000 - (db2Pascal * p + P0) * g001)
+    return (gibbs(n0, n0, n0, SA, t, p) -
+            (db2Pascal * p + P0) * gibbs(n0, n0, n1, SA, t, p))
 
 
 @match_args_return
@@ -150,10 +151,7 @@ def rho_t_exact(SA, t, p):
     2011-03-29. Paul Barker, David Jackett and Trevor McDougal
     """
 
-    n0, n1 = 0, 1
-    rho = 1. / gibbs(n0, n0, n1, SA, t, p)
-
-    return rho
+    return 1. / gibbs(n0, n0, n1, SA, t, p)
 
 
 @match_args_return
@@ -222,12 +220,12 @@ def sigma0_pt0_exact(SA, pt0):
     g08 = x2 * (-3310.49154044839 +
           x * (199.459603073901 +
           x * (-54.7919133532887 +
-          x *  36.0284195611086 -
-          y *  22.6683558512829) +
+          x * 36.0284195611086 -
+          y * 22.6683558512829) +
           y * (-175.292041186547 +
           y * (383.058066002476 +
           y * (-460.319931801257 +
-          y *  234.565187611355)))) +
+          y * 234.565187611355)))) +
           y * (729.116529735046 +
           y * (-860.764303783977 +
           y * (694.244814133268 +
@@ -239,6 +237,7 @@ def sigma0_pt0_exact(SA, pt0):
 
     return 100000000. / (g03 + g08) - 1000.0
 
+
 @match_args_return
 def enthalpy_t_exact(SA, t, p):
     r"""Calculates the specific enthalpy of seawater.
@@ -292,10 +291,8 @@ def enthalpy_t_exact(SA, t, p):
     2011-03-29. David Jackett, Trevor McDougall and Paul Barker.
     """
 
-    n0, n1 = 0, 1
-    g000 = gibbs(n0, n0, n0, SA, t, p)
-    g010 = gibbs(n0, n1, n0, SA, t, p)
-    return g000 - (t + Kelvin) * g010
+    return (gibbs(n0, n0, n0, SA, t, p) -
+            (t + Kelvin) * gibbs(n0, n1, n0, SA, t, p))
 
 
 @match_args_return
@@ -345,7 +342,6 @@ def specvol_t_exact(SA, t, p):
     2011-03-23. David Jackett and Paul Barker.
     """
 
-    n0, n1 = 0, 1
     return gibbs(n0, n0, n1, SA, t, p)
 
 
@@ -405,7 +401,6 @@ def entropy_t_exact(SA, t, p):
     2011-03-29. David Jackett, Trevor McDougall and Paul Barker.
     """
 
-    n0, n1 = 0, 1
     return -gibbs(n0, n1, n0, SA, t, p)
 
 
@@ -454,8 +449,6 @@ def cp_t_exact(SA, t, p):
     2011-03-29. David Jackett, Trevor McDougall and Paul Barker
     """
 
-    n0, n2 = 0, 2
-
     return -(t + Kelvin) * gibbs(n0, n2, n0, SA, t, p)
 
 
@@ -520,12 +513,9 @@ def sound_speed_t_exact(SA, t, p):
     2011-03-29. David Jackett, Paul Barker and Trevor McDougall.
     """
 
-    n0, n1, n2 = 0, 1, 2
-    g020 = gibbs(n0, n2, n0, SA, t, p)
-    g011 = gibbs(n0, n1, n1, SA, t, p)
-    g001 = gibbs(n0, n0, n1, SA, t, p)
-    g002 = gibbs(n0, n0, n2, SA, t, p)
-    return g001 * np.sqrt(g020 / (g011 ** 2 - g020 * g002))
+    return (gibbs(n0, n0, n1, SA, t, p) * np.sqrt(gibbs(n0, n2, n0, SA, t, p) /
+            (gibbs(n0, n1, n1, SA, t, p) ** 2 - gibbs(n0, n2, n0, SA, t, p) *
+            gibbs(n0, n0, n2, SA, t, p))))
 
 
 @match_args_return
@@ -579,13 +569,10 @@ def specvol_anom_t_exact(SA, t, p):
     2011-03-23. Trevor McDougall and Paul Barker
     """
 
-    n0, n1 = 0, 1
-
     pt_zero = pt_from_CT(SSO, 0)
-
     t_zero = pt_from_t(SSO, pt_zero, 0, p)
-
-    return  (gibbs(n0, n0, n1, SA, t, p) - gibbs(n0, n0, n1, SSO, t_zero, p))
+    return (gibbs(n0, n0, n1, SA, t, p) -
+            gibbs(n0, n0, n1, SSO, t_zero, p))
 
 
 @match_args_return
@@ -635,7 +622,6 @@ def chem_potential_relative_t_exact(SA, t, p):
     2011-03-29. Trevor McDougall and Paul Barker
     """
 
-    n0, n1 = 0, 1
     return gibbs(n1, n0, n0, SA, t, p)
 
 
@@ -696,11 +682,9 @@ def internal_energy_t_exact(SA, t, p):
     2011-03-29. Trevor McDougall
     """
 
-    n0, n1 = 0, 1
-    g000 = gibbs(n0, n0, n0, SA, t, p)
-    g010 = gibbs(n0, n1, n0, SA, t, p)
-    g001 = gibbs(n0, n0, n1, SA, t, p)
-    return g000 - (Kelvin + t) * g010 - (db2Pascal * p + P0) * g001
+    return (gibbs(n0, n0, n0, SA, t, p) -
+            (Kelvin + t) * gibbs(n0, n1, n0, SA, t, p) -
+            (db2Pascal * p + P0) * gibbs(n0, n0, n1, SA, t, p))
 
 
 @match_args_return
@@ -757,10 +741,7 @@ def kappa_const_t_exact(SA, t, p):
     2011-03-29. David Jackett, Trevor McDougall and Paul Barker
     """
 
-    n0, n1, n2 = 0, 1, 2
-    g002 = gibbs(n0, n0, n2, SA, t, p)
-    g001 = gibbs(n0, n0, n1, SA, t, p)
-    return -g002 / g001
+    return -gibbs(n0, n0, n2, SA, t, p) / gibbs(n0, n0, n1, SA, t, p)
 
 
 @match_args_return
@@ -816,11 +797,7 @@ def alpha_wrt_t_exact(SA, t, p):
     2011-03-29. David Jackett, Trevor McDougall and Paul Barker
     """
 
-    n0, n1 = 0, 1
-
-    g011 = gibbs(n0, n1, n1, SA, t, p)
-    g001 = gibbs(n0, n0, n1, SA, t, p)
-    return g011 / g001
+    return gibbs(n0, n1, n1, SA, t, p) / gibbs(n0, n0, n1, SA, t, p)
 
 
 @match_args_return
@@ -870,13 +847,8 @@ def isochoric_heat_cap_t_exact(SA, t, p):
     2011-03-29. Trevor McDougall
     """
 
-    n0, n1, n2 = 0, 1, 2
-
-    g020 = gibbs(n0, n2, n0, SA, t, p)
-    g011 = gibbs(n0, n1, n1, SA, t, p)
-    g002 = gibbs(n0, n0, n2, SA, t, p)
-
-    return -(Kelvin + t) * (g020 - g011 * g011 / g002)
+    return (-(Kelvin + t) * (gibbs(n0, n2, n0, SA, t, p) -
+            gibbs(n0, n1, n1, SA, t, p) ** 2 / gibbs(n0, n0, n2, SA, t, p)))
 
 
 @match_args_return
@@ -944,14 +916,9 @@ def kappa_t_exact(SA, t, p):
     2011-03-23. David Jackett, Trevor McDougall and Paul Barker
     """
 
-    n0, n1, n2 = 0, 1, 2
-
-    g020 = gibbs(n0, n2, n0, SA, t, p)
-    g011 = gibbs(n0, n1, n1, SA, t, p)
-    g002 = gibbs(n0, n0, n2, SA, t, p)
-    g001 = gibbs(n0, n0, n1, SA, t, p)
-
-    return (g011 ** 2 - g020 * g002) / (g001 * g020)
+    return ((gibbs(n0, n1, n1, SA, t, p) ** 2 - gibbs(n0, n2, n0, SA, t, p) *
+            gibbs(n0, n0, n2, SA, t, p)) / (gibbs(n0, n0, n1, SA, t, p) *
+            gibbs(n0, n2, n0, SA, t, p)))
 
 
 @match_args_return
@@ -1017,18 +984,21 @@ def SA_from_rho_t_exact(rho, t, p):
     2011-03-28. Trevor McDougall and Paul Barker.
     """
 
-    n0, n1 = 0, 1
     v_lab = np.ones_like(rho) / rho
     v_0 = gibbs(n0, n0, n1, 0, t, p)
     v_120 = gibbs(n0, n0, n1, 120, t, p)
-    SA = 120 * (v_lab - v_0) / (v_120 - v_0)  # Initial estimate of SA.
-    Ior = (SA < 0) | (SA > 120)
-    v_SA = (v_120 - v_0) / 120  # Initial estimate of v_SA, SA derivative of v
+
+    # Initial estimate of SA.
+    SA = 120 * (v_lab - v_0) / (v_120 - v_0)
+    Ior = np.logical_or(SA < 0, SA > 120)
+
+    # Initial estimate of v_SA, SA derivative of v
+    v_SA = (v_120 - v_0) / 120
 
     for k in range(0, 2):
         SA_old = SA
         delta_v = gibbs(n0, n0, n1, SA_old, t, p) - v_lab
-        # Half way through the modified N-R method.
+        # Half way the mod. N-R method (McDougall and Wotherspoon, 2012)
         SA = SA_old - delta_v / v_SA
         SA_mean = 0.5 * (SA + SA_old)
         v_SA = gibbs(n1, n0, n1, SA_mean, t, p)
@@ -1133,12 +1103,12 @@ def t_from_rho_exact(rho, SA, p):
 
     I_salty = alpha_freezing > alpha_limit
 
-    t_diff = 40*ones(size(I_salty)) - t_freezing(I_salty);
+    t_diff = 40. * np.ones_like(I_salty) - t_freezing(I_salty)
 
     top = (rho_40[I_salty] - rho_freezing[I_salty] +
     rho_freezing[I_salty] * alpha_freezing[I_salty] * t_diff)
 
-    a   = top / (t_diff ** 2)
+    a = top / (t_diff ** 2)
     b = -rho_freezing[I_salty] * alpha_freezing[I_salty]
     c = rho_freezing[I_salty] - rho[I_salty]
     sqrt_disc = np.sqrt(b ** 2 - 4 * a * c)
@@ -1201,7 +1171,7 @@ def t_from_rho_exact(rho, SA, p):
         I_quad = ~np.isnan(t_a)
         t[I_quad] = t_a[I_quad]
 
-    I_quad = ~isnan(t_b)
+    I_quad = ~np.isnan(t_b)
     t_multiple[I_quad] = t_b[I_quad]
 
     # After three iterations of this modified Newton-Raphson iteration,
@@ -1209,6 +1179,7 @@ def t_from_rho_exact(rho, SA, p):
 
     return t, t_multiple
 
+
 @match_args_return
 def pot_rho_t_exact(SA, t, p, p_ref=0):
     r"""Calculates potential density of seawater.
@@ -1314,16 +1285,9 @@ def alpha_wrt_CT_t_exact(SA, t, p):
     2011-03-29. Trevor McDougall and Paul Barker
     """
 
-    n0, n1, n2 = 0, 1, 2
-
     pt0 = pt0_from_t(SA, t, p)
-
     factor = -cp0 / ((Kelvin + pt0) * gibbs(n0, n2, n0, SA, t, p))
-
-    g011 = gibbs(n0, n1, n1, SA, t, p)
-    g001 = gibbs(n0, n0, n1, SA, t, p)
-
-    return factor * (g011 / g001)
+    return factor * (gibbs(n0, n1, n1, SA, t, p) / gibbs(n0, n0, n1, SA, t, p))
 
 
 @match_args_return
@@ -1374,15 +1338,9 @@ def alpha_wrt_pt_t_exact(SA, t, p):
     2011-03-29. David Jackett, Trevor McDougall and Paul Barker
     """
 
-    n0, n1, n2 = 0, 1, 2
-
     pt0 = pt0_from_t(SA, t, p)
-
     factor = gibbs(n0, n2, n0, SA, pt0, 0) / gibbs(n0, n2, n0, SA, t, p)
-    g011 = gibbs(n0, n1, n1, SA, t, p)
-    g001 = gibbs(n0, n0, n1, SA, t, p)
-
-    return  factor * (g011 / g001)
+    return factor * (gibbs(n0, n1, n1, SA, t, p) / gibbs(n0, n0, n1, SA, t, p))
 
 
 @match_args_return
@@ -1435,22 +1393,15 @@ def beta_const_CT_t_exact(SA, t, p):
 
     # TODO: Original GSW-V3 re-implements gibbs, check what to do here!
 
-    n0, n1, n2 = 0, 1, 2
-
     pt0 = pt0_from_t(SA, t, p)
 
-    g001 = gibbs(n0, n0, n1, SA, t, p)
-    g110 = gibbs(n1, n1, n0, SA, t, p)
-    g100 = gibbs(n1, n0, n0, SA, pt0, 0)
+    factora = (gibbs(n1, n1, n0, SA, t, p) - gibbs(n1, n0, n0, SA, pt0, 0) /
+               (Kelvin + pt0))
+    factor = (factora / (gibbs(n0, n0, n1, SA, t, p) *
+              gibbs(n0, n2, n0, SA, t, p)))
 
-    factora = g110 - g100 / (Kelvin + pt0)
-    g020 = gibbs(n0, n2, n0, SA, t, p)
-    factor = factora / (g001 * g020)
-
-    g011 = gibbs(n0, n1, n1, SA, t, p)
-    g101 = gibbs(n1, n0, n1, SA, t, p)
-
-    return g011 * factor - g101 / g001
+    return (gibbs(n0, n1, n1, SA, t, p) * factor -
+            gibbs(n1, n0, n1, SA, t, p) / gibbs(n0, n0, n1, SA, t, p))
 
 
 @match_args_return
@@ -1500,23 +1451,17 @@ def beta_const_pt_t_exact(SA, t, p):
     Modifications:
     2011-04-10. Trevor McDougall and Paul Barker
     """
-    # TODO: Original GSW-V3 re-implements gibbs, check what to do here!
-
-    n0, n1, n2 = 0, 1, 2
+    # NOTE: The original Matlab toolbox re-implement some code here.  Why?
 
     pt0 = pt0_from_t(SA, t, p)
 
-    g001 = gibbs(n0, n0, n1, SA, t, p)
-    g110 = gibbs(n1, n1, n0, SA, t, p)
-    factora = g110 - gibbs(n1, n1, n0, SA, pt0, 0)
+    factora = gibbs(n1, n1, n0, SA, t, p) - gibbs(n1, n1, n0, SA, pt0, 0)
 
-    g020 = gibbs(n0, n2, n0, SA, t, p)
-    factor = factora / (g001 * g020)
+    factor = (factora / (gibbs(n0, n0, n1, SA, t, p) *
+              gibbs(n0, n2, n0, SA, t, p)))
 
-    g011 = gibbs(n0, n1, n1, SA, t, p)
-    g101 = gibbs(n1, n0, n1, SA, t, p)
-
-    return g011 * factor - g101 / g001
+    return (gibbs(n0, n1, n1, SA, t, p) * factor -
+            gibbs(n1, n0, n1, SA, t, p) / gibbs(n0, n0, n1, SA, t, p))
 
 
 @match_args_return
@@ -1567,11 +1512,7 @@ def beta_const_t_exact(SA, t, p):
     2011-03-29. David Jackett, Trevor McDougall and Paul Barker
     """
 
-    n0, n1 = 0, 1
-
-    g101 = gibbs(n1, n0, n1, SA, t, p)
-    g001 = gibbs(n0, n0, n1, SA, t, p)
-    return -g101 / g001
+    return -gibbs(n1, n0, n1, SA, t, p) / gibbs(n0, n0, n1, SA, t, p)
 
 
 @match_args_return
@@ -1798,10 +1739,7 @@ def adiabatic_lapse_rate_t_exact(SA, t, p):
     2011-03-29. Trevor McDougall and Paul Barker
     """
 
-    n0, n1, n2 = 0, 1, 2
-    g011 = gibbs(n0, n1, n1, SA, t, p)
-    g020 = gibbs(n0, n2, n0, SA, t, p)
-    return - g011 / g020
+    return -gibbs(n0, n1, n1, SA, t, p) / gibbs(n0, n2, n0, SA, t, p)
 
 
 @match_args_return
@@ -1848,18 +1786,43 @@ def osmotic_coefficient_t_exact(SA, t, p):
     UNESCO (English), 196 pp.
 
     Modifications:
-    2011-04-01. Trevor McDougall and Paul Barker
+    2011-04-01. Trevor McDougall and Paul Barker.
+    2012-11-15. Trevor McDougall and Paul Barker.
     """
 
-    n0 = 0
+    SA = np.maximum(SA, 0)
+    k = M_S / R
+    part = k * (1000 - SA) / (Kelvin + t)
 
-    # Molality of seawater in mol/kg.
-    molal = molality_from_SA(SA)
-    part = molal * R * (Kelvin + t)
+    x2 = sfac * SA
+    x = np.sqrt(x2)
+    y = t * 0.025
+    # Note that the input pressure (p) is sea pressure in units of dbar.
+    z = p / db2Pascal
 
-    SAzero = 0
-    g000 = gibbs(n0, n0, n0, SAzero, t, p)
-    return  (g000 - chem_potential_water_t_exact(SA, t, p)) / part
+    oc = (7.231916621570606e1, 1.059039593127674e1, -3.025914794694813e1,
+          5.040733670521486e1, -4.074543321119333e1, 1.864215613820487e1,
+          -3.022566485046178, -6.138647522851840, 1.353207379758663e1,
+          -7.316560781114737, 1.829232499785750, -5.358042980767074e-1,
+          -1.705887283375562, -1.246962174707332e-1, 1.228376913546017,
+          1.089364009088042e-2, -4.264828939262248e-1, 6.213127679460041e-2,
+          2.481543497315280, -1.363368964861909, -5.640491627443773e-1,
+          1.344724779893754, -2.180866793244492, 4.765753255963401,
+          -5.726993916772165, 2.918303792060746, -6.506082399183509e-1,
+          -1.015695507663942e-1, 1.035024326471108, -6.742173543702397e-1,
+          8.465642650849419e-1, -7.508472135244717e-1, -3.668086444057845e-1,
+          3.189939162107803e-1, -4.245629194309487e-2)
+
+    tl = (oc[0] + oc[1] * y + x * (oc[2] + x * (oc[3] + x * (oc[4] + x *
+         (oc[5] + oc[6] * x))) + y * (oc[7] + x * (oc[8] + x *
+         (oc[9] + oc[10] * x)) + y * (oc[11] + oc[12] * x + y * (oc[13] +
+         oc[14] * x + y * (oc[15] + x * (oc[16] + oc[17] * y))))) + z *
+         (oc[18] + x * (oc[19] + oc[20] * y + oc[21] * x) + y * (oc[22] + y *
+         (oc[23] + y * (oc[24] + oc[25] * y))) + z * (oc[26] + oc[27] * x + y *
+         (oc[28] + oc[29] * y) + z * (oc[30] + oc[31] * x + y * (oc[32] +
+         oc[33] * y) + oc[34] * z)))))
+
+    return tl * part
 
 
 @match_args_return
@@ -1959,7 +1922,6 @@ def t_maxdensity_exact(SA, p):
     2011-04-03. Trevor McDougall and Paul Barker
     """
 
-    n0, n1 = 0, 1
     # The temperature increment for calculating the gibbs_PTT derivative.
     dt = 0.001
     t = 3.978 - 0.22072 * SA  # The initial guess of t_maxden.
@@ -1968,7 +1930,7 @@ def t_maxdensity_exact(SA, p):
     for Number_of_iterations in range(0, 3):
         t_old = t
         gibbs_PT = gibbs(n0, n1, n1, SA, t_old, p)
-        # This is half way through the modified method
+        # Half way through the mod. method (McDougall and Wotherspoon, 2012)
         t = t_old - gibbs_PT / gibbs_PTT
         t_mean = 0.5 * (t + t_old)
         gibbs_PTT = (gibbs(n0, n1, n1, SA, t_mean + dt, p) -
@@ -2021,15 +1983,15 @@ def osmotic_pressure_t_exact(SA, t, pw):
     Modifications:
     2011-05-26. Trevor McDougall and Paul Barker
     """
-
+    SA = np.maximum(SA, 0)
     gibbs_pure_water = gibbs(0, 0, 0, 0, t, pw)
 
     # Initial guess of p, in dbar.
     p = pw + 235.4684
 
     # Initial guess of df/dp.
-    df_dp = -db2Pascal * (gibbs(0, 0, 1, SA, t, p) -
-                          SA * gibbs(1, 0, 1, SA, t, p))
+    df_dp = -db2Pascal * (gibbs(n0, n0, n1, SA, t, p) -
+                          SA * gibbs(n1, n0, n1, SA, t, p))
 
     for Number_of_iterations in range(0, 2):
         p_old = p
