@@ -1,5 +1,7 @@
 # Quick value test
+import os
 import gsw
+import numpy as np
 from gsw.utilities import Dict2Struc
 
 # Read data file with check value profiles
